<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Adapter;

use \Zend_Rest_Client;
use \Zend_Http_Client;
use \Magento\Framework\DataObject as DataObject;
use \FiloBlu\MappConnector\Helper\ExceptionHelper;

/**
 * Class RestAdapter
 * @package FiloBlu\MappConnector\Adapter
 */
class RestAdapter extends Zend_Rest_Client implements MappAdapterInterface
{
    /**
     * Client
     *
     * @var
     */
    protected $_client;

    /**
     * Rest Config
     *
     * @var
     */
    protected $_config;

    /**
     * System Config
     *
     * @var
     */
    protected $_systemconfig;

    /**
     * FileLogger Helper
     *
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_fileLogger;
    
    /**
     * RestAdapter constructor.
     */
    public function __construct(){

    }

    /**
     * Inits the Adapter
     *
     * @throws \Zend_Http_Client_Exception
     */
    function _init(){
        $config = [
            "baseurl" => $this->_systemconfig->getApiBaseUrl(),
            "api_prefix" => $this->_systemconfig->getApiPrefix(),
            "username" => $this->_systemconfig->getApiUsername(),
            "password" => $this->_systemconfig->getApiPassword(),
            "authentication" => Zend_Http_Client::AUTH_BASIC
        ];

        $this->setConfig($config);
        $this->setUri($this->_config['baseurl']);
        $this->getHttpClient()->setAuth($this->_config['username'], $this->_config['password'], $this->_config['authentication']);
        $this->getHttpClient()->setHeaders(
            array(
                'Content-type: application/json',
                'Accept: application/json',
            )
        );
    }

    /**
     * Sets the Adapter Config
     *
     * @param $config
     */
    function setConfig($config){
        $this->_config = $config;
    }

    /**
     * Gets the Adapter Config
     *
     * @return mixed
     */
    function getConfig(){
        return $this->_config;
    }

    /**
     * Sets the System Config
     *
     * @param $config
     */
    public function setSystemConfig($config) {
        $this->_systemconfig = $config;
    }

    /**
     * Gets the System Config
     *
     * @return mixed
     */
    public function getSystemConfig() {
        return $this->_systemconfig;
    }

    /**
     * Sets the File Logger
     *
     * @param $fileLogger
     */
    public function setFileLogger($fileLogger) {
        $this->_fileLogger = $fileLogger;
    }

    /**
     * Makes the request
     *
     * @param DataObject $data
     * @return mixed
     * @throws \Exception
     */
    function makeRequest(DataObject $data){
        $endpoint = $data->getData('endpoint');
        $method = $data->getData('method');
        $params = $data->getData('params');

        try {
            if (!isset($endpoint)) {
                $this->_fileLogger->error('Invalid Rest Endpoint', [], __METHOD__ . ':' . __LINE__);
            }

            $methods = array('post', 'get', 'delete', 'put');

            if (in_array(strtolower($method), $methods)) {
                $response = $this->{'rest' . ucfirst(strtolower($method))}($this->_config['api_prefix'] . $endpoint, $params);
                return $response;
            }
            else {
                $this->_fileLogger->error('Invalid Rest Method', [], __METHOD__ . ':' . __LINE__);
            }
        }
        catch (\Exception $e){
            $this->_fileLogger->error($e->getMessage(), [], __METHOD__ . ':' . __LINE__);
        }

    }

    /**
     * Override Zend_client _performPost
     *
     * @param mixed $method
     * @param null $data
     * @return \Zend_Http_Response
     * @throws \Zend_Http_Client_Exception
     */
    protected function _performPost($method, $data = null)
    {
        $client = self::getHttpClient();
        if(isset($data['post'])) {
            if (is_string($data['post'])) {
                $client->setRawData($data['post'],'application/json');
            } elseif (is_array($data['post']) || is_object($data['post'])) {
                $client->setParameterPost((array)$data['post']);
            }
        }

        if (isset($data['get'])){
            $client->setParameterGet((array) $data['get']);
        }

        return $client->request($method);
    }


}