<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class Prepare
 * @package FiloBlu\MappConnector\Console\Command
 */
class Prepare extends Command
{

    /**
     * @var \FiloBlu\MappConnector\Model\Jobs
     */
    protected $_jobs;

    /**
     * @var \FiloBlu\MappConnector\Model\Mapp
     */
    protected $_mapp;

    /**
     * @var \FiloBlu\MappConnector\Model\MappCSV
     */
    protected $_mappCSV;

    /**
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_loggerHelper;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $_state;

    /**
     * @var \FiloBlu\MappConnector\Model\Config
     */
    protected $_config;

    /**
     * @var array
     */
    protected $_entities = ['customer','subscriber','order','product','quote'];

    /**
     * Prepare constructor.
     * @param \FiloBlu\MappConnector\Model\Mapp $mapp
     * @param \FiloBlu\MappConnector\Model\MappCSV $mappCSV
     * @param \FiloBlu\MappConnector\Model\Jobs $jobs
     * @param \Magento\Framework\App\State $state
     * @param \FiloBlu\MappConnector\Helper\FileLoggerHelper $loggerHelper
     * @param \FiloBlu\MappConnector\Model\Config $config
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\Mapp $mapp,
        \FiloBlu\MappConnector\Model\MappCSV $mappCSV,
        \FiloBlu\MappConnector\Model\Jobs $jobs,
        \Magento\Framework\App\State $state,
        \FiloBlu\MappConnector\Helper\FileLoggerHelper $loggerHelper,
        \FiloBlu\MappConnector\Model\Config $config
    )
    {
        $this->_jobs = $jobs;
        $this->_mapp = $mapp;
        $this->_mappCSV = $mappCSV;
        $this->_state = $state;
        $this->_loggerHelper = $loggerHelper;
        $this->_config = $config;
        parent::__construct();
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    )
    {
        //Area Code Fix
        try {
            $this->_state->setAreaCode(\Magento\Framework\App\Area::AREA_GLOBAL);
        }
        catch (\Exception $e) {
            //Area code is already set
        }

        $entity = strtolower(trim($input->getArgument('entity')));
        $t = strtolower(trim($input->getArgument('type')));
        $m = strtolower(trim($input->getArgument('mode')));

        //Prepare Jobs
        $this->prepareJobs($entity,$t,$m);

    }

    public function prepareJobs($entity = null, $t = null, $m = null){
        $type = 'incremental';
        if(!empty($t) && in_array($t,['incremental','full','fullincremental'])) {
            $type = $t;
        }

        $mode = 'ftp';
        if(!empty($m) && in_array($m,['sftp','rest'])) {
            $mode = $m;
            if($mode == 'rest' && !in_array($entity,['customer','subscriber'])) {
                $output->writeln("\n-- WARNING: rest type is allowed only on customer and subscriber --\n");
                return;
            }
        }


        //If entity is specified and exist process entity
        if(!empty($entity) && $entity != 'all') {
            if(in_array($entity, $this->_entities)) {
                $this->_mappCSV->setJobType($entity);
                $this->_mappCSV->prepareExport($type,$mode);
            }
            else {
                $output->writeln("WARNING: The entity specified ".$entity." doesn't exists");
            }
        }
        // Else process all entities
        else {
            foreach ($this->_entities as $entity_item) {
                $this->_mappCSV->setJobType($entity_item);
                $this->_mappCSV->prepareExport($type,$mode);
            }
        }

        // To backup rest jobs on ftp
        $this->_convertCompletedRestToFtpJobs();
    }

    protected function _convertCompletedRestToFtpJobs() {
        if($this->_config->getConfigValue('mappconnector/general/enable_ftp_backup_rest') == 1){
            $ids = $this->_jobs->getAllCompletedRestIds();
            $this->_jobs->massUpdateJob($ids,'waiting','sftp');

            $this->_loggerHelper->debug('Converted to SFTP '.count($ids).' Rest processes');
        }
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName("mapp:prepare");
        $this->setDescription("Prepare full export");
        // Next line will add a new required parameter to our script
        $this->addArgument('entity', InputArgument::OPTIONAL, __('Please specify an Entity'));
        $this->addArgument('type', InputArgument::OPTIONAL, __('Please specify type of export'));
        $this->addArgument('mode', InputArgument::OPTIONAL, __('Please specify mode of export'));
        parent::configure();
    }
}
